/*  exampleScalarProbe.c
 *  Sample code for probing a dataset of scalars.
 *  Camille Goudeseune, 1/11/99
 */

#include "vssClient.h"


/*** Interface between wand buttons and .aud file *************************/

float audfileFloats[3];

void MapValueToAudfile(float value)
{
	audfileFloats[0] = 0.2;                 /* loudness */
	audfileFloats[1] = 6.0 + 2.0 * value;   /* pitch */
	audfileFloats[2] = 0.02;                /* width */

}

void StartProbe(float value)
{
	MapValueToAudfile(value);
	AUDupdateSimple("ButtonDown", 3, audfileFloats);
}

void ChangeData(float value)
{
	MapValueToAudfile(value);
	AUDupdateSimple("WandMove", 3, audfileFloats);
}

void EndProbe()
{
	AUDupdateSimple("ButtonUp", 0, NULL);
}


/*** Dummy dataset-access function. ***************************************/

float GetValue()
{
	/* Bounded random walk. */
	static float value = 0.5;
	value += drand48()-0.5 + drand48()-0.5;
	if (value < 0.0)
		value = 0.0;
	else if (value > 1.0)
		value = 1.0;
	return value;
}


/**************************************************************************/

int main()
{
	 int i;

/*** Open communication link to server. ***/

	if (!BeginSoundServer())	{
		printf("Error: couldn't find a running copy of VSS.\n");
		return -1;
	}

/*** Start making a sound. ***/

	AUDinit("continuous_probe.aud");

/*** Pretend that the user was probing a dataset. ***/

	/* wand button down */
	StartProbe(GetValue());

	/* wand moves */
	for (i=0; i<10; i++)
		{
		sginap(30);
		ChangeData(GetValue());
		}
	sginap(50);

	/* wand button up */
	EndProbe();

	/* do it once more */
	sginap(50);
	StartProbe(GetValue());
	sginap(30);
	ChangeData(GetValue());
	sginap(30);
	EndProbe();

/*** Stop the sound, and end communication with server. ***/

	EndSoundServer();
	return 0;
}
