/*  exampleVectorProbe.c
 *  Sample code for probing a dataset of vectors.
 *  Camille Goudeseune, 1/11/99
 */

#include "vssClient.h"


/*** Interface between wand buttons and .aud file *************************/

float audfileFloats[3];

void MapValuesToAudfile(float* values)
{
	audfileFloats[0] = values[0]*.2 + .2;               /* loudness */
	audfileFloats[1] = 6.0 + values[3] + values[4];     /* pitch */
	audfileFloats[2] = 0.2 * (values[1] + values[2]);   /* width */

}

void StartProbe(float* values)
{
	MapValuesToAudfile(values);
	AUDupdateSimple("ButtonDown", 3, audfileFloats);
}

void ChangeData(float* values)
{
	MapValuesToAudfile(values);
	AUDupdateSimple("WandMove", 3, audfileFloats);
}

void EndProbe()
{
	AUDupdateSimple("ButtonUp", 0, NULL);
}


/*** Dummy dataset-access function. ***************************************/

float* GetValues(float* array)
{
	int i;
	for (i=0; i<5; i++)
	{
		/* Bounded random walks. */
		static float x[5] = {0};

		x[i] += drand48()-0.5 + drand48()-0.5;
		if (x[i] < 0.0)
			x[i] = 0.0;
		else if (x[i] > 1.0)
			x[i] = 1.0;
		array[i] = x[i];
	}
	return array;
}


/**************************************************************************/

int main()
{
	 int i;
	 float values[5];

/*** Open communication link to server. ***/

	if (!BeginSoundServer())	{
		printf("Error: couldn't find a running copy of VSS.\n");
		return -1;
	}

/*** Start making a sound. ***/

	AUDinit("continuous_probe.aud");

/*** Pretend that the user was probing a dataset. ***/

	/* wand button down */
	StartProbe(GetValues(values));

	/* wand moves */
	for (i=0; i<10; i++)
		{
		sginap(30);
		ChangeData(GetValues(values));
		}
	sginap(50);

	/* wand button up */
	EndProbe();

	/* do it once more */
	sginap(50);
	StartProbe(GetValues(values));
	sginap(30);
	ChangeData(GetValues(values));
	sginap(30);
	EndProbe();

/*** Stop the sound, and end communication with server. ***/

	EndSoundServer();
	return 0;
}
