#include <cave_ogl.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <unistd.h>

#include "vssClient.h"
int fAudio = 1;
int hAud = -1; // handle to .aud file


#define numspheres 11
float rate = 0.15;
float speed[numspheres][3];

typedef struct {
  float spheres[numspheres][6];
  int collisions[numspheres];
  int collisionsPrev[numspheres];
  int bang[numspheres];
  int bangPrev[numspheres];
  int bangPrev2[numspheres];
  float totalspeed;
} shared_stuff;

shared_stuff* pshared;

void initVideo()
{
  float lightModelAmbient[] = { 0.05, 0.05, 0.05, 1.0 };
  glLightModelfv (GL_LIGHT_MODEL_AMBIENT, lightModelAmbient);

  float lightDiffuse[] = { 0.7, 0.7, 0.7 };
  float lightPosition[] = { 0.5, 0.5, 0.1, 0 };

  glLightfv (GL_LIGHT0, GL_DIFFUSE,  lightDiffuse);
  glLightfv (GL_LIGHT0, GL_POSITION, lightPosition);

  // Activate the light model.
  glEnable (GL_LIGHTING);

  // Activate the lights.
  glEnable (GL_LIGHT0);
}

float randomshellcoord(void)
{
	float _ = drand48()* 2. - 1.;
	if (_<0) _ -= 4.5; else _ += 4.5;
	return _;
}

void my_draw(void)
{
  int i;

  // Clear the z-buffer and set the screen to black
  glClearDepth (1.0);
  glClearColor (0.0, 0.0, 0.0, 1.0);
  glClear (GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

  const float colorColl[] = {1.0f, 0.3f, 0.05f};
  const float colorBang[] = {1.0f, 0.2f, 0.01f};
  const float colorMightBang[] = {0.7f, 0.2f, 0.9f};

  const float    glowing[] = { .3, .3, .3, 1. };
  const float    GLOWing[] = { .6, .6, .6, 1. };
  const float notglowing[] = { .0, .0, .0, 1. };
  glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, glowing);
  for (i=0; i<numspheres; i++) {
    float radius = pshared->spheres[i][5];
	if (i==0) radius *= 4;
    if (pshared->bang[i] || pshared->bangPrev[i] || pshared->bangPrev2[i])
	  {
      glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, colorBang);
	  radius *= 1.3;
	  }
    else if (pshared->collisions[i])
	  {
      glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, colorColl);
	  radius *= 1.5;
	  }
    else
	  {
	  const float colorDull[] = {
		0.7f*i/numspheres,
		1.0f*i/numspheres,
		1.0f*(numspheres-i)/numspheres
		};
      glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, i==0? colorMightBang : colorDull);
	  }

    glPushMatrix();
    glTranslatef (pshared->spheres[i][0], pshared->spheres[i][1], pshared->spheres[i][2]);
    GLUquadric *quad = gluNewQuadric();
    gluSphere(quad, radius, 14, 14);
    glPopMatrix();

  }

	// Draw lines shooting out from "current" collisions.
    glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, GLOWing);
	{
	const float colorSpark[] = { 1., 1., 1. };
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, colorSpark);
	for (i=0; i<numspheres; i++)
		{
		if (pshared->bang[i] || pshared->bangPrev[i] || pshared->bangPrev2[i] ||
			pshared->collisions[i] || pshared->collisionsPrev[i])
			{
			glPushMatrix();
			glTranslatef (pshared->spheres[i][0], pshared->spheres[i][1], pshared->spheres[i][2]);
			glBegin(GL_TRIANGLES);
			for (int j=0; j<40; j++)
				{
				// 2 other points of the fan: one random on spherical shell, another near it.
				float p1[3] = { randomshellcoord(), randomshellcoord(), randomshellcoord() };
				float p2[3] = { p1[0] + drand48()*.3, p1[1] + drand48()*.3, p1[2] + drand48()*.3 };
				glVertex3f(0,0,0);
				glVertex3fv(p1);
				glVertex3fv(p2);
				}
			glEnd();
			glPopMatrix();
			}
		}
	}
  glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, notglowing);

	// Draw walls, so viewer can predict when a bang will happen
	// by watching balls hit the wall.
	{
	float colorWalls[] = {0.4f, 0.0f, 0.3f};
	float expando, expandO;
	for (expando = 7, expandO = 7.5;
		expando > -4.9;
		expando -= 1.1, expandO -= .02,
			colorWalls[0] = .4 - colorWalls[0],
			colorWalls[2] = .3 - colorWalls[2])
		{
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, colorWalls);
		glBegin(GL_QUADS);

		glVertex3f(-5-expandO, -0-expando, -5-expando);
		glVertex3f(-5-expandO, 10+expando, -5-expando);
		glVertex3f(-5-expandO, 10+expando,  6);
		glVertex3f(-5-expandO, -0-expando,  6);

		glVertex3f( 5+expandO, -0-expando, -5-expando);
		glVertex3f( 5+expandO, 10+expando, -5-expando);
		glVertex3f( 5+expandO, 10+expando,  6);
		glVertex3f( 5+expandO, -0-expando,  6);

		glVertex3f( 5+expando, -0-expandO, -5-expando);
		glVertex3f(-5-expando, -0-expandO, -5-expando);
		glVertex3f(-5-expando, -0-expandO,  6);
		glVertex3f( 5+expando, -0-expandO,  6);

		glVertex3f( 5+expando, 10+expandO, -5-expando);
		glVertex3f(-5-expando, 10+expandO, -5-expando);
		glVertex3f(-5-expando, 10+expandO,  6);
		glVertex3f( 5+expando, 10+expandO,  6);

		glVertex3f( 5+expando, 10+expando, -5-expandO);
		glVertex3f(-5-expando, 10+expando, -5-expandO);
		glVertex3f(-5-expando, -0-expando, -5-expandO);
		glVertex3f( 5+expando, -0-expando, -5-expandO);

		glVertex3f( 5+expando, 10+expando,  6 + 0.5);
		glVertex3f(-5-expando, 10+expando,  6 + 0.5);
		glVertex3f(-5-expando, -0-expando,  6 + 0.5);
		glVertex3f( 5+expando, -0-expando,  6 + 0.5);

		glEnd();
		}
	}
}

void DoSphereDynamics()
{
	int i;

	/* update position of spheres */
	for (i=0; i<numspheres; i++)
		{
		pshared->spheres[i][0] += speed[i][0];
		pshared->spheres[i][1] += speed[i][1];
		pshared->spheres[i][2] += speed[i][2];
		}

	/* update speeds of spheres, to stop them from going too far */
	const float expando = 7;
	for (i=0; i<numspheres; i++)
		{
		int fHitWall = 0;
		if (pshared->spheres[i][0] < -5-expando)
			{
			fHitWall = 1;
			pshared->spheres[i][0] = -5-expando;
			speed[i][0] =   drand48() * rate + rate/5.;
			}
		if (pshared->spheres[i][0] >  5+expando)
			{
			fHitWall = 1;
			pshared->spheres[i][0] = 5+expando;
			speed[i][0] = -(drand48() * rate + rate/5.);
			}
		if (pshared->spheres[i][1] < -0-expando)
			{
			fHitWall = 1;
			pshared->spheres[i][1] = -0-expando;
			speed[i][1] =   drand48() * rate + rate/5.;
			}
		if (pshared->spheres[i][1] > 10+expando)
			{
			fHitWall = 1;
			pshared->spheres[i][1] = 10+expando;
			speed[i][1] = -(drand48() * rate + rate/5.);
			}
		if (pshared->spheres[i][2] < -5-expando)
			{
			fHitWall = 1;
			pshared->spheres[i][2] = -5-expando;
			speed[i][2] =   drand48() * rate + rate/5.;
			}
		if (pshared->spheres[i][2] >  6)
			{
			fHitWall = 1;
			pshared->spheres[i][2] = 6;
			speed[i][2] = -(drand48() * rate + rate/5.);
			}

		int fBang = i==0 && fHitWall; // be louder than the rest
		pshared->bang[i] = fBang;
		if (fHitWall)
			AUDupdate(hAud, fBang?"bang":"wall", 5, pshared->spheres[i]);
		}
	for (i=0; i<numspheres; i++)
		{
		pshared->bangPrev2[i] = pshared->bangPrev[i];
		pshared->bangPrev[i] = pshared->bang[i];
		}

//printf("%.1f %.1f %.1f       %.1f %.1f %.1f\n",
//	pshared->spheres[0][0], pshared->spheres[0][1], pshared->spheres[0][2],
//	speed[0][0], speed[0][1], speed[0][2]);;;;

	/* compute average speed of all spheres */
	pshared->totalspeed = 0.;
	for (i=0; i<numspheres; i++)
		{
		pshared->totalspeed +=
		    fabs(speed[i][0]) + fabs(speed[i][1]) + fabs(speed[i][2]);
		}
}

/* test for collisions */
void DoCollisionTest()
{
	int i,j;

	for (i=0; i<numspheres; i++)
		pshared->collisions[i] = 0;

	for (i=0; i<numspheres; i++)
	for (j=i+1; j<numspheres; j++)
		{
		if (fabs(pshared->spheres[i][0] - pshared->spheres[j][0]) +
		    fabs(pshared->spheres[i][1] - pshared->spheres[j][1]) +
		    fabs(pshared->spheres[i][2] - pshared->spheres[j][2]) < 1.3)
			{
			pshared->collisions[i] = 1;
			pshared->collisions[j] = 1;
			}
		}
	for (i=0; i<numspheres; i++)
		if (pshared->collisions[i] && !pshared->collisionsPrev[i])
			{
			AUDupdate(hAud, "collision", 5, pshared->spheres[i]);
			speed[i][0] += drand48() * rate * .07;
			speed[i][1] += drand48() * rate * .07;
			speed[i][2] += drand48() * rate * .07;
			}
	for (i=0; i<numspheres; i++)
		pshared->collisionsPrev[i] = pshared->collisions[i];
}

void init_shared_data()
{
	int i;

	for (i=0; i<numspheres; i++)
		{
		pshared->spheres[i][0] = drand48() * 6. - 3.;
		pshared->spheres[i][1] = drand48() * 6. + 2.;
		pshared->spheres[i][2] = drand48() * 6. - 3.;
		pshared->spheres[i][3] = 350.+200.*i/numspheres; // pitch of collision sound
		pshared->spheres[i][4] = 175.+ 40.*i/numspheres; // pitch of wall-collision sound
		pshared->spheres[i][5] = .15; // radius
		pshared->collisions[i] = 0;
		pshared->collisionsPrev[i] = 0;
		}

	rate = 0.15;

	/* set initial random speed of spheres */
	for (i=0; i<numspheres; i++)
		{
		speed[i][0] = drand48() * rate - rate/2.;
		speed[i][1] = drand48() * rate - rate/2.;
		speed[i][2] = drand48() * rate - rate/2.;
		}
}

void main(int argc, char**argv)
{
  printf("Hit F1/F2 to speed up/slow down, esc to quit, R to reset.\n");

  CAVEConfigure(&argc,argv,NULL);
  if (fAudio)
	{
	if (!BeginSoundServer())
		fAudio = 0;
	else if ((hAud = AUDinit("spheres.aud")) < 0)
		{
		EndSoundServer();
		fAudio = 0;
		}
	}
  pshared = (shared_stuff*) CAVEMalloc(sizeof(shared_stuff));
  init_shared_data();
  CAVEInit();
  CAVEInitApplication(initVideo, 0);
  CAVEDisplay(my_draw, 0);
  while (!CAVEgetbutton(CAVE_ESCKEY)) {
    DoSphereDynamics();
    DoCollisionTest();
    if (CAVEgetbutton(CAVE_RKEY))
      init_shared_data(); /*SOUND*/
    if (CAVEgetbutton(CAVE_F1KEY) && rate < .8)
      rate *= 1.02; /*SOUND*/
    if (CAVEgetbutton(CAVE_F2KEY) && rate > .006)
      rate /= 1.02; /*SOUND*/
    sginap(1);
  }
  if (fAudio)
	{
	AUDterminate(hAud);
	EndSoundServer();
	}
  CAVEExit();
}
